#include "mouse.hpp"

Mouse::Mouse()
{
	display_status = MOUSE_OFF;
	
	// terminate program if no mouse installed
	if (MouseCommand(MOUSE_RESET) == 0) {
	  GraphicsHandler->PutString(BRIGHT_RED,
	    100, 100, "Mouse driver not installed.");
	  exit(1);
	}
}

void Mouse::Show()
{
	if (display_status == MOUSE_ON)
	  return;
	display_status = MOUSE_ON;
	MouseCommand(MOUSE_SHOW);
}

void Mouse::Hide()
{

	if (display_status == MOUSE_OFF)
	  return;
	display_status = MOUSE_OFF;
	MouseCommand(MOUSE_HIDE);
}

int Mouse::Status(int* x, int* y)
{
	// read the button and (X, Y) status
	MouseCommand(MOUSE_STATUS);

	// save the X coordinate of the mouse
	*x = results. x. cx;

	// save the Y coordinate of the mouse,
	// after converting to a right-handed
	// coordinate system, in which the lower left
	// corner of the screen is the origin
	*y = RightHandedCartesian(results. x. dx);

	// return the button status
	return results. x. bx;
}

int Mouse::MouseCommand(MouseFunctionCode function,
			 unsigned int bx,
			 unsigned int cx,
			 unsigned dx)
{
	REGS command;

	command. x. ax = function;
	command. x. bx = bx;
	command. x. cx = cx;
	command. x. dx = dx;
	int86(MOUSE_INTERRUPT, &command, &results);
	return results. x. ax;
}


void Mouse::SetPosition(int x, int y)
{
	// convert screen coordinates to a right
	// handed system
	y = RightHandedCartesian(y);
	MouseCommand(MOUSE_SET_POSITION, 0, x, y);
}

void Mouse::ButtonPressedInfo(int button,
			     int* status,
			     int* count,
			     int* x,
			     int* y)
{
	ButtonInfo(MOUSE_PRESSED_INFO,
		   button, status, count, x, y);
}

void Mouse::ButtonReleasedInfo(int button,
			     int* status,
			     int* count,
			     int* x,
			     int* y)
{
	ButtonInfo(MOUSE_RELEASED_INFO,
		   button, status, count, x, y);
}

void Mouse::ButtonInfo(MouseFunctionCode command,
		       int button,
		       int* status,
		       int* count,
		       int* x,
		       int* y)

{
	if ( (button == MOUSE_LEFT_BUTTON) ||
	     (button == MOUSE_RIGHT_BUTTON) )

	     // convert to Microsoft format
	     button--;

	else {
	  // invalid button parameter: clear variables
	  *status = 0;
	  *count = 0;
	  *x = 0;
	  *y = 0;
	  return;
	}

	*status = MouseCommand(command, button);
	*count = results. x. bx;
	*x = results. x. cx;
	*y = RightHandedCartesian(results. x. dx);
}

void Mouse::SetXRange(int x1, int x2)
{
	MouseCommand(MOUSE_X_RANGE, 0, x1, x2);
}

void Mouse::SetYRange(int y1, int y2)
{
	MouseCommand(MOUSE_Y_RANGE, 0,
		     RightHandedCartesian(y1),
		     RightHandedCartesian(y2) );
}


int Mouse::LeftButtonDown()
{
	int x, y;
	return (Status(&x, &y) & MOUSE_LEFT_BUTTON) ?
		TRUE : FALSE;
}

int Mouse::RightButtonDown()
{
	int x, y;
	return (Status(&x, &y) & MOUSE_RIGHT_BUTTON) ?
		TRUE : FALSE;
}

int Mouse::BothButtonsDown()
{
	return LeftButtonDown() && RightButtonDown();
}


int Mouse::RightHandedCartesian(int y)
{
	return GraphicsHandler->ScreenHeight() - y;
}
